################################################################################################################
########### Normaliztion and plotting of melting curve data (HRM) from StepONE Plus Real-Time System ########### 
################################################################################################################

library("ggplot2")
library("plyr")
library("reshape2")
library("gridExtra")
library("shiny")
library("agricolae")
start_temp <- 40
temp_int <- 0.1
choose.dir.mac <- function() {
  system("osascript -e 'tell app \"RStudio\" to POSIX path of (choose folder with prompt \"please choose a folder:\")' > /tmp/R_folder",intern = FALSE, ignore.stderr = TRUE)
  p <- system("cat /tmp/R_folder && rm -f /tmp/R_folder", intern = TRUE)
  return(ifelse(length(p), p, NA))
}

{if (Sys.info()[['sysname']] == "Windows")
  dd <-  choose.dir(getwd(),"please choose a folder")
 else
   dd <- choose.dir.mac()
}
setwd(dd)


sample <- read.csv("sample.csv")


files_multi <- list.files(pattern = ".*multicomponentdata.*.txt",recursive = TRUE)
dat_multi <- read.csv(files_multi, header=F, skip=9,sep="")
dat <- dat_multi
dat <- delete.na(dat,"greater")
names(dat) <- c("well","temp","sybr","rox")
dat$nor_in <- dat$sybr/dat$rox

split_data <- split(dat, list(dat$well))
for (n in names(split_data)){
  split_data[[n]]$temp <- start_temp + split_data[[n]]$temp*temp_int
  split_data[[n]]$nor_in <- (split_data[[n]]$nor_in - min(split_data[[n]]$nor_in))/(max(split_data[[n]]$nor_in)-min(split_data[[n]]$nor_in))*100 
  if (length(split_data[[n]]) > 1)
    nor_dat <- do.call(rbind,split_data)
}
a <- data.frame(well=nor_dat$well, name=sample[match(nor_dat$well, sample$well), 2])
nor_dat <- cbind(a,nor_dat)



files_derivative <- list.files(pattern = ".*meltregionderivativedata.*.txt",recursive = TRUE)
dat_derivative <- read.csv(files_derivative, header=F, skip=9,sep="")
dat_derivative <- dat_derivative[,-c(1,3,4,5)]

melt <- melt(dat_derivative)
names(melt) <- c("well","temp","value")
melt$temp <- as.numeric(melt$temp)

split_data <- split(melt, list(melt$well))
for (n in names(split_data)){
  split_data[[n]]$temp <- start_temp + split_data[[n]]$temp*temp_int
  if (length(split_data[[n]]) > 1)
    nor_melt <- do.call(rbind,split_data)
}

nor_melt1 <- nor_melt
a1 <- data.frame(well=nor_melt$well, name=sample[match(nor_melt$well, sample$well), 2])
nor_melt1 <- cbind(a1,nor_melt)

server <- function(input, output){
  nor_melt2 <- reactive({
    nor_melt2 <- nor_melt1[nor_melt1$name %in% input$name,]
    nor_melt2
  })
  output$mainPlot <- renderPlot({
    nor_melt <- nor_melt2()
    nor_melt <- nor_melt[nor_melt$temp > input$temp[1] & nor_melt$temp < input$temp[2], ]
    if (input$data_ave==1){
      nor_melt$name <- paste0(nor_melt$well,"_",nor_melt$name)
    }
    nor_melt <- ddply(nor_melt,.(name),transform,value=value/max(value))
    nor_melt_nor <- ddply(nor_melt,.(name,temp),summarise,value=mean(value),sd=sd(value))
    nor_melt_nor <- ddply(nor_melt_nor,.(name),transform,value=value/max(value))
    nor_melt_max <- ddply(nor_melt_nor,.(name),subset,value==max(value))
    nor_melt_max$x <- input$temp[2]
    nor_melt_max$y <- 1-as.numeric(paste0("0.",1:nrow(nor_melt_max)))
    p2 <- ggplot(data = nor_melt_nor,aes(temp,value,colour= factor(name)))+ geom_line(aes(),size=1) + theme_bw() + theme(legend.title=element_blank())
    p2 <- p2 + labs(x = "Temperature", y = "Derivative Reporter (-R) ")
    if (input$radio==1)
      p2 <- p2 + geom_vline(aes(xintercept=temp,colour=factor(name)),data=nor_melt_max)
    if (input$radio==2)
      p2 <- p2 + geom_text(data=nor_melt_max, aes(x=x,y=y,colour=factor(name),label=temp),size=6,check_overlap = TRUE)
    ratio_values <- 4/3
    ratio_display <- (max(nor_melt_nor$temp)-min(nor_melt_nor$temp))/(max(nor_melt_nor$value)-min(nor_melt_nor$value))
    p2 <-p2 +coord_fixed(ratio_display/ratio_values)
    p2
    output$plot1 <- downloadHandler(
      filename= function(){
        paste("melt","pdf",sep=".")
      },
      content= function(file){
        pdf(file)
        plot(p2)
        dev.off()
      })
    print(p2)
  })
  output$mainPlot1 <- renderPlot({
    nor_dat <- nor_dat[nor_dat$name %in% input$name,]
    nor_dat <- nor_dat[nor_dat$temp > input$temp[1] & nor_dat$temp < input$temp[2], ]
    if (input$data_ave==1){
      nor_dat$name <- paste0(nor_dat$well,"_",nor_dat$name)
    }
    nor_dat_nor <- ddply(nor_dat,.(name,temp),summarise,nor_in=mean(nor_in),sd1=sd(nor_in))
    ratio_values <- 4/3
    ratio_display <- (max(nor_dat_nor$temp)-min(nor_dat_nor$temp))/(max(nor_dat_nor$nor_in)-min(nor_dat_nor$nor_in))
    
    p <- ggplot(data = nor_dat_nor,aes(temp,nor_in,colour= factor(name)))+ geom_line(aes(),size=1) + theme_bw()+ theme(legend.title=element_blank())
    p <- p + annotate("text", label= input$tex, x = input$xl, y = input$yl, size = 5, colour = "blue")
    p <- p + labs(x = "Temperature", y = "Normalized fluorescence intensity")
    p <- p + coord_fixed(ratio_display/ratio_values)
    p
    output$plot <- downloadHandler(
      filename= function(){
        paste("normalized","pdf",sep=".")
      },
      content= function(file){
        pdf(file)
        plot(p)
        dev.off()
      })
    print(p)  
  })
  
}


ui <- shinyUI(fluidPage(
  title = 'Examples of DataTables',
  sidebarLayout(
    sidebarPanel(
      radioButtons("data_ave", label = h6("show single data or average data",style = "color:blue"),
                   choices = list("single data" = 1, "average data" = 2), 
                   selected = 2),
      checkboxGroupInput('name', label = h6('choose your samples:',style = "color:blue"),
                         levels(nor_melt1$name), selected = levels(nor_melt1$name)[1]),
      sliderInput("temp", 
                  label = h6("minmun and maximun of temperature",style = "color:blue"),
                  min = 40, max = 100, value = c(40, 90), step = 1),
      radioButtons("radio", label = h6("show plot with line or Tm",style = "color:blue"),
                   choices = list("line" = 1, "Tm" = 2, "w/o" = 3), 
                   selected = 3)
    ),
    mainPanel(
      plotOutput("mainPlot"),
      downloadButton(outputId = "plot1",label = "download melt plot"),
      plotOutput("mainPlot1"),
      downloadButton(outputId = "plot",label = "download normalized plot")
      
    )
  )))

shinyApp(ui = ui, server = server)
